/**
 * <p>Ketai Library for Android: http://ketai.org</p>
 *
 * <p>KetaiBluetooth wraps the Android Bluetooth RFCOMM Features:
 * <ul>
 * <li>Enables Bluetooth for sketch through android</li>
 * <li>Provides list of available Devices</li>
 * <li>Enables Discovery</li>
 * <li>Allows writing data to device</li>
 * </ul>
 * <p>Updated: 2012-05-18 Daniel Sauter/j.duran</p>
 */

//required for BT enabling on startup
import android.content.Intent;
import android.os.Bundle;

import ketai.net.bluetooth.*;
import ketai.ui.*;
import ketai.net.*;

import oscP5.*;

KetaiBluetooth bt;
String info = "";
KetaiList klist;
PVector remoteMouse = new PVector();

ArrayList<String> devicesDiscovered = new ArrayList();
boolean isConfiguring = true;
String UIText;

byte[] message = new byte[1];

//********************************************************************
// The following code is required to enable bluetooth at startup.
//********************************************************************
void onCreate(Bundle savedInstanceState) {
  super.onCreate(savedInstanceState);
  bt = new KetaiBluetooth(this);
}

void onActivityResult(int requestCode, int resultCode, Intent data) {
  bt.onActivityResult(requestCode, resultCode, data);
}

//********************************************************************

void setup()
{   
  orientation(PORTRAIT);
  background(78, 93, 75);
  stroke(255);
  textSize(24);

  //start listening for BT connections
  bt.start();

  UIText =  "d - discover devices\n" +
    "b - make this device discoverable\n" +
    "c - connect to device\n     from discovered list.\n" +
    "p - list paired devices\n" +
    "i - Bluetooth info";
}

void draw()
{
  if (isConfiguring)
  {
    ArrayList<String> names;
    background(78, 93, 75);

    //based on last key pressed lets display
    //  appropriately
    if (key == 'i')
      info = getBluetoothInformation();
    else
    {
      if (key == 'p')
      {
        info = "Paired Devices:\n";
        names = bt.getPairedDeviceNames();
      }
      else
      {
        info = "Discovered Devices:\n";
        names = bt.getDiscoveredDeviceNames();
      }

      for (int i=0; i < names.size(); i++)
      {
        info += "["+i+"] "+names.get(i).toString() + "\n";
      }
    }
    text(UIText + "\n\n" + info, 5, 90);
  }
  else
  {
    background(78, 93, 75);
    pushStyle();
    fill(255);
    ellipse(mouseX, mouseY, 20, 20);
    fill(0, 255, 0);
    stroke(0, 255, 0);
    ellipse(remoteMouse.x, remoteMouse.y, 20, 20);    
    popStyle();
  }

  if (mousePressed == true) {
    if(300 <= mouseX && mouseX <= 400 && 400 <= mouseY && mouseY <= 500)
    {
      // 'F'
      message[0] = 70;
      bt.broadcast(message);
      fill(255,255,0);
      rect(300, 400, 100, 100);
      fill(50,50,50);
      rect(300, 600, 100, 100);
      fill(50,50,50);
      rect(200, 500, 100, 100);
      fill(50,50,50);
      rect(400, 500, 100, 100);     
    }
    if(300 <= mouseX && mouseX <= 400 && 600 <= mouseY && mouseY <= 700)
    {
      // 'B'
      message[0] = 66;
      bt.broadcast(message);
      fill(50,50,50);
      rect(300, 400, 100, 100);
      fill(255,255,0);
      rect(300, 600, 100, 100);
      fill(50,50,50);
      rect(200, 500, 100, 100);
      fill(50,50,50);
      rect(400, 500, 100, 100);  
    }  
    if(200 <= mouseX && mouseX <= 300 && 500 <= mouseY && mouseY <= 600)
    {
      // 'L'
      message[0] = 76;
      bt.broadcast(message);
      fill(50,50,50);
      rect(300, 400, 100, 100);
      fill(50,50,50);
      rect(300, 600, 100, 100);
      fill(255,255,0);
      rect(200, 500, 100, 100);
      fill(50,50,50);
      rect(400, 500, 100, 100);  
    }  
    if(400 <= mouseX && mouseX <= 500 && 500 <= mouseY && mouseY <= 600)
    {
      // 'R'
      message[0] = 82;
      bt.broadcast(message);
      fill(50,50,50);
      rect(300, 400, 100, 100);
      fill(50,50,50);
      rect(300, 600, 100, 100);
      fill(50,50,50);
      rect(200, 500, 100, 100);
      fill(255,255,0);
      rect(400, 500, 100, 100);  
    }       
  }
  else
  {
    fill(0,0,0);
    rect(300, 400, 100, 100);
    fill(0,0,0);
    rect(300, 600, 100, 100);
    fill(0,0,0);
    rect(200, 500, 100, 100);
    fill(0,0,0);
    rect(400, 500, 100, 100);     
  }
 

  drawUI();
}


//Call back method to manage data received
void onBluetoothDataEvent(String who, byte[] data)
{
  if (isConfiguring)
    return;

  //KetaiOSCMessage is the same as OscMessage
  //   but allows construction by byte array
  KetaiOSCMessage m = new KetaiOSCMessage(data);
  if (m.isValid())
  {
    if (m.checkAddrPattern("/remoteMouse/"))
    {
      if (m.checkTypetag("ii"))
      {
        remoteMouse.x = m.get(0).intValue();
        remoteMouse.y = m.get(1).intValue();
      }
    }
  }
}

String getBluetoothInformation()
{
  String btInfo = "Server Running: ";
  btInfo += bt.isStarted() + "\n";
  btInfo += "Discovering: " + bt.isDiscovering() + "\n";
  btInfo += "Device Discoverable: "+bt.isDiscoverable() + "\n";
  btInfo += "\nConnected Devices: \n";

  ArrayList<String> devices = bt.getConnectedDeviceNames();
  for (String device: devices)
  {
    btInfo+= device+"\n";
  }

  return btInfo;
}
